﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using MetadataExtractor;
using MetadataExtractor.Formats.Exif;
using MetadataExtractor.Formats.QuickTime;
using Microsoft.Extensions.Configuration;
using Serilog;
using ShellProgressBar;
using Directory = System.IO.Directory;

namespace PhotoRenamer
{
    internal class Renamer
    {
        private readonly string _sourcePath;
        private readonly string _targetPath;

        public Renamer(IConfiguration configuration)
        {
            _sourcePath = Path.GetFullPath(configuration["Source"]);
            _targetPath = Path.GetFullPath(configuration["Target"]);
            Log.Information($"Source path: {_sourcePath}");
            Log.Information($"Target path: {_targetPath}");
        }

        public int Run()
        {
            var files = Directory.GetFiles(_sourcePath, "*", SearchOption.AllDirectories);
            var options = new ProgressBarOptions
            {
                ForegroundColor = ConsoleColor.Yellow,
                ForegroundColorDone = ConsoleColor.DarkGreen,
                BackgroundColor = ConsoleColor.DarkGray,
                BackgroundCharacter = '\u2593'
            };
            using var progressBar = new ProgressBar(files.Length, "Copying files", options);
            foreach (var file in files)
            {
                progressBar.Tick();
                try
                {
                    var directories = ImageMetadataReader.ReadMetadata(file);

                    var dateTime = GetDateTimeFromExif(directories) ?? GetDateTimeFromMp4(directories);
                    if (dateTime is null) continue;
                    var dayFolder = Path.Combine(
                        _targetPath,
                        dateTime.Value.Year.ToString(),
                        dateTime.Value.Month.ToString("D2"),
                        dateTime.Value.Day.ToString("D2"));
                    if (!Directory.Exists(dayFolder)) Directory.CreateDirectory(dayFolder);
                    var destination = Path.Combine(dayFolder, Path.GetFileName(file));
                    if (File.Exists(destination)) continue;
                    File.Copy(file, destination);
                }
                catch (ImageProcessingException)
                {
                    //silently ignore
                }
            }

            return 0;
        }

        private static DateTime? GetDateTimeFromExif(IEnumerable<MetadataExtractor.Directory> directories)
        {
            return directories
                .OfType<ExifSubIfdDirectory>()
                .FirstOrDefault()?.GetDateTime(ExifDirectoryBase.TagDateTimeOriginal);
        }
        
        private static DateTime? GetDateTimeFromMp4(IEnumerable<MetadataExtractor.Directory> directories)
        {
            return directories
                .OfType<QuickTimeMovieHeaderDirectory>()
                .FirstOrDefault()?.GetDateTime(QuickTimeMovieHeaderDirectory.TagCreated);
        }
    }
}